/*****************************************************************************
 *                                                                           
 *  Copyright (c) 2006 by SANTEC/TUDOR www.santec.tudor.lu                   
 *                                                                           
 *                                                                           
 *  This library is free software; you can redistribute it and/or modify it  
 *  under the terms of the GNU Lesser General Public License as published    
 *  by the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                      
 *                                                                           
 *  This software is distributed in the hope that it will be useful, but     
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
 *  Lesser General Public License for more details.                          
 *                                                                           
 *  You should have received a copy of the GNU Lesser General Public         
 *  License along with this library; if not, write to the Free Software      
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
 *                                                                           
 *****************************************************************************/

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URLDecoder;
import java.security.CodeSource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

/**
 * Helper class for an ImageJ lauchner application that sets classpath and memory.
 * @author Johannes Hermen johannes.hermen(at)tudor.lu
 *
 */
public class ImageJStarter {

	
	private static Process ps;

	/**
	 * @param args
	 */
	public static void main(String[] args) {


		String BASEDIR = getBaseDir() + "/";
		String CLASSPATH = BASEDIR +"ij.jar"+File.pathSeparator+ BASEDIR +"plugins"+File.pathSeparator;
		System.out.println("BASEDIR:"+BASEDIR);
		System.out.println("CLASSPATH:"+CLASSPATH);
		try {
			List files = getFileListing(new File(BASEDIR + "/lib"));
		    Iterator filesIter = files.iterator();
		    while( filesIter.hasNext() ){
		    	Object next = filesIter.next();
		      System.out.println("add: " +next );
		      CLASSPATH += next + File.pathSeparator;
		    }
		    System.out.println("CLASSPATH:"+CLASSPATH);
		    System.out.println("java -cp "+CLASSPATH+" ij.ImageJ");
		    Properties properties = new Properties();
		    try {
		        properties.load(new FileInputStream(BASEDIR + "starter.properties"));
		    } catch (IOException e) {
		    	e.printStackTrace();
		    }
		    String memory = properties.getProperty("memory","-Xmx512m");
		    String[] arguments = {"java",memory, "-cp",CLASSPATH, "ij.ImageJ"};
		    System.out.println(memory);
		    ps = Runtime.getRuntime().exec(arguments, null, new File(BASEDIR));

		    // read system.out
		    new Thread() {
		    	public void run() {
		    		try {
		    			String text;
		    		    PrintWriter out = new PrintWriter(System.out);
		    			BufferedReader in = new BufferedReader(new InputStreamReader(ps.getInputStream()));
		    			        while ((text = in.readLine()) != null) {
		    			          out.println(text); out.flush();
		    			        }
		    		    } catch (IOException e) {
		    		    }
		    	}
		    }.start();
		    
		    //	read system.err
		    new Thread() {
		    	public void run() {
		    		try {
		    			String text;
		    		    PrintWriter err = new PrintWriter(System.err);
		    			BufferedReader errin = new BufferedReader(new InputStreamReader(ps.getErrorStream()));
		    			        while ((text = errin.readLine()) != null) {
		    			          err.println(text); err.flush();
		    			        }
		    		    } catch (IOException e) {
		    		    }
		    	}
		    }.start();
		    
		    
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * Recursively walk a directory tree and return a List of all Files found;
	 * the List is sorted using File.compareTo.
	 * 
	 * @param aStartingDir
	 *            is a valid directory, which can be read.
	 */
	static public List getFileListing(File aStartingDir)
			throws FileNotFoundException {
		List result = new ArrayList();

		File[] filesAndDirs = aStartingDir.listFiles();
		File file = null;
		for (int i = 0; i < filesAndDirs.length; i++) {
			file = filesAndDirs[i];
			if (file.isFile() && file.getName().toLowerCase().endsWith(".jar")) {
				result.add(file); // always add, even if directory
			}
			if (!file.isFile()) {
				// must be a directory
				// recursive call!
				List deeperList = getFileListing(file);
				result.addAll(deeperList);
			}

		}
		Collections.sort(result);
		return result;
	}
	
	/**
	   * Detects the location of the jar-file
	   * @return the absolut path to the jar-file
	   */
	  static public String getBaseDir(){
	    String     l_baseDir = null;

	    // create url from jarfile location
	    CodeSource source = ImageJStarter.class.getProtectionDomain().getCodeSource();
	    String url = source.getLocation().getPath();
	    url = URLDecoder.decode(url);
	    // extract path from url
	    l_baseDir = (new File(url)).getParent();

	    return l_baseDir;
	  }
	  
}
