package lu.tudor.santec.dicom;

/*****************************************************************************
 *                                                                           
 *  Copyright (c) 2006 by SANTEC/TUDOR www.santec.tudor.lu                   
 *                                                                           
 *                                                                           
 *  This library is free software; you can redistribute it and/or modify it  
 *  under the terms of the GNU Lesser General Public License as published    
 *  by the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                      
 *                                                                           
 *  This software is distributed in the hope that it will be useful, but     
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
 *  Lesser General Public License for more details.                          
 *                                                                           
 *  You should have received a copy of the GNU Lesser General Public         
 *  License along with this library; if not, write to the Free Software      
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
 *                                                                           
 *****************************************************************************/

import ij.ImagePlus;
import ij.io.FileInfo;
import ij.process.ByteProcessor;
import ij.process.ImageProcessor;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.io.File;
import java.io.FileNotFoundException;

import javax.swing.JOptionPane;

import loci.formats.IFormatReader;
import loci.formats.in.DicomReader;
import loci.plugins.Util;
import lu.tudor.santec.dicom.gui.header.DicomHeader;

/**
 * This class is capable to open DICOM images using the JAI Library. This is needed if
 * you like to open compressed (lossless jpeg,...) DICOM images.
 * 
 * 
 * @author Andreas Jahnen andreas.jahnen(at)tudor.lu
 *
 */
public class DicomOpener{
	
	public static ImagePlus loadImage(File p_File, Component parentComponent) throws FileNotFoundException {

		// check if file is there:
		if (!p_File.canRead()) {
			System.out.println("Unable to load file... " + p_File);
			return null;
		}

		// b) read file name:
		ImagePlus image = null;
		try {
			
			IFormatReader r = new DicomReader();
			r.setId(p_File.getAbsolutePath());
			ImageProcessor ip2 = Util.openProcessors(r, 0)[0];
			image = new ImagePlus(p_File.getName(), ip2);

//			BufferedImage bf = reader.openImage(0);	
//			image = new ImagePlus(p_File.getName(), bf);
//			ImageProcessor ip = image.getProcessor();
//			ip = ip.convertToShort(false);
//			ImagePlus image2 = new ImagePlus (p_File.getName(), ip);
			
			DicomHeader dh = new DicomHeader(p_File);
			image.setProperty("Info", dh.toString());
			image.setProperty(DicomHeader.class.getSimpleName(), dh);
			
			FileInfo fi = new FileInfo();
			fi.directory = p_File.getParent()+ File.separator;
			fi.fileName = p_File.getName();
			image.setFileInfo(fi);
			
			return image;
			
		} catch (Exception e) {
			System.out.println("Unable to open image file: "
					+ p_File.getAbsolutePath() + "\n" + e.getMessage());
//			e.printStackTrace();
			
			if (parentComponent != null) {
			    JOptionPane.showMessageDialog(parentComponent,
				    "Unable to open image file: "
					+ p_File.getAbsolutePath() + "\n" + e.getMessage(),
				    "Error loading Image",
				    JOptionPane.ERROR_MESSAGE);

			    
			}
			
			DicomHeader dh = new DicomHeader(p_File);

			byte[] bArr = new byte[1024*1024];
			for (@SuppressWarnings("unused") byte b : bArr) {
			    b = Byte.MAX_VALUE;
			}
			ImageProcessor iProz = new ByteProcessor(1024, 1024, bArr, null);
			
			iProz.setColor(Color.WHITE);
			iProz.setAntialiasedText(true);
			iProz.setFont(new Font("Arial", Font.BOLD, 30));
			iProz.drawString("Unable to open image:\n" +
				"Modality="+dh.getHeaderStringValue("0008,0060")+"\n"+ 
				(dh.containsHeaderTag("0002,0010")?"Transfer Syntax="+dh.getHeaderStringValue("0002,0010")+"\n":"")+
				"file: " + p_File.getName() + "\n" + 
				e.getMessage(), 150, 400);
			
			ImagePlus ip = new ImagePlus(p_File.getName(), iProz);
			ip.setProperty("Info", dh.toString());
			ip.setProperty(DicomHeader.class.getSimpleName(), dh);

			FileInfo fi = new FileInfo();
			fi.directory = p_File.getParent()+ File.separator;
			fi.fileName = p_File.getName();
			ip.setFileInfo(fi);
			return ip;
		}
	}
}
