package lu.tudor.santec.dicom.demoapps;

import ij.ImagePlus;
import ij.process.Blitter;
import ij.process.ByteProcessor;
import ij.process.ColorProcessor;
import ij.process.ImageProcessor;

import java.awt.image.IndexColorModel;
import java.io.File;
import java.io.IOException;

import org.dcm4che2.data.DicomElement;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.data.Tag;
import org.dcm4che2.io.DicomInputStream;
import org.dcm4che2.io.StopTagInputHandler;

public class OverlayExtractor {

    public OverlayExtractor() {
	
    }
    
    public static boolean hasOverlayImage(File f) {
	DicomInputStream in;
	try {
	    in = new DicomInputStream(f);
	    in.setHandler(new StopTagInputHandler(Tag.PixelData));
	    DicomObject dcmobj =  in.readDicomObject();
	    in.close();
	    return hasOverlayImage(dcmobj);
	} catch (IOException e) {
	    e.printStackTrace();
	}
	return false;
    }
    
    public static boolean hasOverlayImage(DicomObject dcmobj) {
	return dcmobj.contains(Tag.OverlayData);
    }
    
    public static ImagePlus extractOverlay(File f) {
	DicomInputStream in;
	try {
	    in = new DicomInputStream(f);
	    in.setHandler(new StopTagInputHandler(Tag.PixelData));
	    DicomObject dcmobj =  in.readDicomObject();
	    in.close();
	    return extractOverlay(dcmobj);
	} catch (IOException e) {
	    e.printStackTrace();
	}
	return null;
    }
    
    public static ImagePlus extractOverlay(DicomObject dcmobj) {
	try {
            
            if (dcmobj.contains(Tag.OverlayData)) {
        	DicomElement olData = dcmobj.get(Tag.OverlayData);
        	
        	int rows = dcmobj.get(Tag.Rows).getInt(false);
        	int cols = dcmobj.get(Tag.Columns).getInt(false);
        	
        	byte[] data = olData.getBytes();
        	byte[] byteData = new byte[data.length*8];
        	
        	int j = 0;
        	for (int i = 0; i < data.length; i++) {
        	    byte b = data[i];
        	    if ((b & 0x01) > 0 ) byteData[j++]=(byte) 0; else byteData[j++]= (byte) 255;
        	    if ((b & 0x02) > 0) byteData[j++]=(byte) 0; else byteData[j++]= (byte) 255;
        	    if ((b & 0x04) > 0) byteData[j++]=(byte) 0; else byteData[j++]= (byte) 255;
        	    if ((b & 0x08) > 0) byteData[j++]=(byte) 0; else byteData[j++]= (byte) 255;
        	    if ((b & 0x10) > 0) byteData[j++]=(byte) 0; else byteData[j++]= (byte) 255;
        	    if ((b & 0x20) > 0) byteData[j++]=(byte) 0; else byteData[j++]= (byte) 255;
        	    if ((b & 0x40) > 0) byteData[j++]=(byte) 0; else byteData[j++]= (byte) 255;
        	    if ((b & 0x80) > 0) byteData[j++]=(byte) 0; else byteData[j++]= (byte) 255;
		}
        	byte[] cmap = {(byte)0, (byte) 255, (byte) 0, (byte)255, (byte)0, (byte) 0, (byte) 0, (byte)0 };
        	ImageProcessor ip = new ByteProcessor(cols,rows, byteData, 
        		new IndexColorModel(1,2,cmap,0,true));
        	
        	return new ImagePlus("overlay",ip);
            }
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return null;
    }
    
    
    public static ImagePlus createOverlayImage(ImagePlus orig, DicomObject dcmobj) {
	ImagePlus overlay = extractOverlay(dcmobj);
//	new ImageJ();
	ColorProcessor cp = (ColorProcessor) orig.getProcessor().convertToRGB();
	ColorProcessor cp2 = (ColorProcessor) overlay.getProcessor().convertToRGB();
//	new ImagePlus("conv", cp2).show();
	cp.copyBits(cp2, 0, 0, Blitter.COPY_ZERO_TRANSPARENT);
//	new ImagePlus("add", cp).show();
	return new ImagePlus("img+overlay", cp);
    }
    
    
    /**
     * @param args
     */
    public static void main(String[] args) {
	OverlayExtractor.extractOverlay(new File("/home/hermenj/9d9accdd"));
//	OverlayExtractor.extractOverlay(new File("/media/ETIAM/images/im510"));
    }

}
